#' GetDataSummaryArtifacts
#'
#' Produces plots and artifacts for the data summary page
#'
#' @param inputData Input data as data.table. Required
#'
#' @return list of objects
#'
#' @examples
#' \dontrun{
#' GetDataSummaryArtifacts(inputData)
#' }
#'
#' @export
GetDataSummaryArtifacts <- function(inputData)
{
  inputData <- copy(inputData)

  missPlot <- PlotMissingnessSummary(inputData)

  # New stuff
  inputData[, ":="(
    NotificationTime = DateOfNotificationYear + 1/4 * DateOfNotificationQuarter,
    DiagnosisTime = DateOfDiagnosisYear + 1/4 * DateOfDiagnosisQuarter
  )]
  inputData[, VarX := 4 * (NotificationTime - DiagnosisTime)]
  inputData[VarX < 0, VarX := NA]

  # Info to display
  missingCount <- inputData[, c(
    NotificationTime = sum(is.na(NotificationTime)),
    DiagnosisTime = sum(is.na(DiagnosisTime)),
    VarX = sum(is.na(VarX))
  )]

  # Observed delay distribution
  par(mfrow = c(1, 2))
  plot(density(inputData$VarX, na.rm = TRUE, from = -1, to = 30, adjust = 4),
       main = "Density of Reporting Delay",
       xlab = "Delay in quarters of the year",
       ylab = "Proportion reported with the delay")
  abline(v = quantile(inputData$VarX, probs = 0.95, na.rm = TRUE),
         col = "red",
         lty = 2,
         lwd = 3)
  text(quantile(inputData$VarX, probs = 0.95, na.rm = TRUE) + 0.5,
       0.1,
       paste("95% of cases reported \nby", quantile(inputData$VarX, probs = 0.95, na.rm = TRUE), "quarters"),
       pos = 4,
       col = "red")

  shortData <- inputData[DateOfDiagnosisYear < max(DateOfDiagnosisYear) - 5]
  plot(density(shortData$VarX, na.rm = TRUE, from = -1, to = 30, adjust = 4),
       main = "Density of Reporting Delay \nLast 5 years deleted",
       xlab = "Delay in quarters of the year",
       ylab = "Proportion reported with the delay")
  abline(v = quantile(shortData$VarX, probs = 0.95, na.rm = TRUE), col = "red", lty = 2, lwd = 3)
  text(quantile(shortData$VarX, probs = 0.95, na.rm = TRUE) + 0.5,
       0.1,
       paste("95% of cases reported \nby", quantile(shortData$VarX, probs = 0.95, na.rm = TRUE), "quarters"),
       pos = 4,
       col = "red")
  delayDensityPlot <- recordPlot()

  # Mean delay plot
  meanDelay <- inputData[,
                         .(MeanDelay = mean(VarX, na.rm = TRUE)),
                         by = .(NotificationTime)]
  meanDelay[, rsd := RollingApply(MeanDelay, 10, sd)]
  meanDelayCurve <- meanDelay[, as.data.table(lowess(NotificationTime, MeanDelay))]
  meanDelayUpper <- meanDelay[, as.data.table(lowess(NotificationTime, MeanDelay + 1.96 * rsd))]

  meanDelayPlot <- ggplot() +
    geom_point(data = meanDelay, aes(NotificationTime, MeanDelay)) +
    geom_line(data = meanDelayCurve, aes(x, y, color = "Smoothed mean reporting delay"), size = 0.5) +
    geom_line(data = meanDelayUpper, aes(x, y, color = "Expected upper bound"), linetype = "dashed", size = 0.5) +
    labs(colour = "Datasets", x = "xxx", y = "yyy") +
    scale_color_manual(values = c("blue", "red")) +
    theme(text = element_text(size = 14)) +
    ggtitle("Reporting delay by notification quarter") +
    xlab("Notification time in quarters of the year") +
    ylab("Mean reporting delay")

  return(list(MissPlot = missPlot,
              DelayDensityPlot = delayDensityPlot,
              MeanDelayPlot = meanDelayPlot))
}
